==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-workflow/src/main/java/de/fzj/unicore/ucc/workflow/SubmitWorkAssignment.java <==
package de.fzj.unicore.ucc.workflow;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Random;

import javax.security.auth.x500.X500Principal;
import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.chemomentum.common.ws.IServiceOrchestrator;
import org.chemomentum.workassignment.xmlbeans.GetWorkAssignmentStatusRequestDocument;
import org.chemomentum.workassignment.xmlbeans.GetWorkAssignmentStatusResponseDocument;
import org.chemomentum.workassignment.xmlbeans.StatusType;
import org.chemomentum.workassignment.xmlbeans.SubmitWorkAssignmentRequestDocument;
import org.chemomentum.workassignment.xmlbeans.WorkAssignmentType;
import org.chemomentum.workassignment.xmlbeans.WorkDocument.Work;
import org.chemomentum.workflow.callback.xmlbeans.ExecutionResourcesDocument.ExecutionResources;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.actions.Resolve;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.ucc.util.Builder;
import de.fzj.unicore.ucc.util.FileDownloader;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import eu.unicore.security.util.client.IClientProperties;
import eu.unicore.security.xfireutil.client.SecuredXFireClientFactory;

/**
 * submit a work assignment to the UNICORE service orchestrator (broker)
 * 
 * @author schuller
 */
public class SubmitWorkAssignment extends WorkflowSystemSubmission implements IServiceInfoProvider{

	protected String workflowName;

	protected IServiceOrchestrator wsc;

	protected EndpointReferenceType servorchEPR;

	protected EndpointReferenceType jobEPR;

	protected String waID;

	protected boolean async=false;

	@Override
	public void process(){
		super.process();

		workflowName=getCommandLine().getOptionValue(OPT_NAME);
		if(getCommandLine().hasOption(OPT_MODE)){
			async=true;
		}
		waID=WSUtilities.newUniqueID();
		try{
			findSite();
			initBuilder(waID);
			run();
		}catch(Exception e){
			error("",e);
			endProcessing(1);
		}
	}

	protected void findSite(){
		String addr=null;
		try{
			List<EndpointReferenceType>available=registry.listAccessibleServices(IServiceOrchestrator.PORT);
			for(EndpointReferenceType epr: available){
				if (siteName!=null){
					addr=epr.getAddress().getStringValue();
					if(addr.contains(siteName)){
						servorchEPR=epr;
						setReceiver();
						wsc=new SecuredXFireClientFactory(securityProperties).createPlainWSProxy(IServiceOrchestrator.class, addr);
						verbose("Using service <"+addr+"> at requested site <"+siteName+">");
						return;
					}
				}
			}
			int i=0;
			if(available.size()==0){
				message("No service orchestrator service available!");
				endProcessing(1);
			}
			else{
				//select one
				i=new Random().nextInt(available.size());
				servorchEPR=available.get(i);
				setReceiver();
				addr=servorchEPR.getAddress().getStringValue();
				wsc=new SecuredXFireClientFactory(securityProperties).createPlainWSProxy(IServiceOrchestrator.class, addr);
			}
			verbose("Selected workflow service at "+addr);
		}
		catch(Exception e){
			error("Can't find a workflow service.",e);
			endProcessing();
		}
	}

	private void setReceiver(){
		String dn=null;
		try{
			dn=WSUtilities.extractServerIDFromEPR(servorchEPR);
			if(dn!=null){
				securityProperties.getETDSettings().setReceiver(new X500Principal(dn));
			}
		}catch(IllegalArgumentException e){
			//dn is invalid
			try{
				logger.warn("Invalid server name <"+dn+"> found in EPR "+servorchEPR.getAddress().getStringValue()+". Not an X500 name.");
			}catch(NullPointerException e1){}
		}

	}

	protected void run() throws Exception{
		//Submit work assignment
		SubmitWorkAssignmentRequestDocument in=SubmitWorkAssignmentRequestDocument.Factory.newInstance();
		WorkAssignmentType wa=in.addNewSubmitWorkAssignmentRequest().addNewWorkAssignment();
		Work work=wa.addNewWork();
		work.setJobDefinition(builder.getJob().getJobDefinition());
		wa.setParent("Submitted_by_UCC");
		wa.setId(waID);
		if(builder.getImports().size()>0){
			createWorkflowDataStorage();
			wa.setStorageEPR(storageEPR);
			try{
				uploadLocalData(builder, waID);
			}catch(IOException ex){
				error("Can't upload local files.",ex);
				endProcessing(1);
			}
		}
		
		logger.debug("Submitting "+String.valueOf(in));
		
		wsc.submitWorkAssignment(in);

		if(!async){
			
			try{
				waitForFinish();
			}catch(Exception ex){
				error("Work assignment did not finish successfully",ex);
				endProcessing(1);
			}
			//get output files
			if(builder.getExports().size()>0){
				for(FileDownloader exp: builder.getExports()){
					performExportToLocal(exp);
				}
			}
		}
	}

	protected void performExportToLocal(FileDownloader e){
		String name=waID+"_"+e.getTo();
		File out=new File(name);
		if(!out.isAbsolute()){
			out=new File(output,name);
		}
		try{
			Location l=Resolve.resolve(e.getFrom(), registry, securityProperties, this);
			EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
			epr.addNewAddress().setStringValue(l.getSmsEpr());
			StorageClient sms=new StorageClient(l.getSmsEpr(),epr,securityProperties);
			FileDownloader e1=new FileDownloader(l.getName(),out.getAbsolutePath(),e.getMode());
			e1.perform(sms, this);
		}
		catch(Exception ex){
			error("Could not perform export "+e.getFrom()+"->"+e.getTo(),ex);
		}
	}

	protected void waitForFinish()throws Exception{
		verbose("Waiting for work assignment to finish...");
		StatusType.Enum s=null;
		boolean havePrintedJobEPR=false;
		GetWorkAssignmentStatusResponseDocument res=null;
		do{
			GetWorkAssignmentStatusRequestDocument gs=GetWorkAssignmentStatusRequestDocument.Factory.newInstance();
			gs.addNewGetWorkAssignmentStatusRequest().setId(waID);
			res=wsc.getWorkAssignmentStatus(gs);
			StatusType.Enum newStatus=res.getGetWorkAssignmentStatusResponse().getStatusResult();
			if(!newStatus.equals(s)){
				s=newStatus;
				message(String.valueOf(s));
			}
			ExecutionResources er=res.getGetWorkAssignmentStatusResponse().getExecutionResources();
			if(er!=null){
				jobEPR=er.getJob();
				if(jobEPR!=null && !havePrintedJobEPR){
					message("Job submitted as: "+er.getJob().getAddress().getStringValue());
					havePrintedJobEPR=true;
				}
			}
			if(StatusType.FAILURE.equals(s) || StatusType.SUCCESS.equals(s))break;
			Thread.sleep(2000);
		}while(true);
		message("Work assignment finished: "+s);
	}


	private void initBuilder(String uniqueID) throws Exception {
		if(getCommandLine().getArgs().length<1){
			error("Please supply the name of the job file,", new IllegalArgumentException());
			endProcessing(1);
		}
		String filename=getCommandLine().getArgs()[1];
		verbose("Loading job from <"+filename+">");
		FileInputStream fis=new FileInputStream(new File(filename).getAbsolutePath());
		ByteArrayOutputStream bos=new ByteArrayOutputStream();
		int b=0;
		while( (b=fis.read())!=-1){
			bos.write(b);
		}
		String job=bos.toString();
		if(fixFilenames){
			job=writeNewIDs(job, uniqueID);
		}
		builder=new Builder(job);
	}


	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_MODE_LONG)
				.withDescription("Run asynchronously")
				.isRequired(false)
				.create(OPT_MODE)
		);


	}

	protected void writeIDFile(EndpointReferenceType epr){
		try{
			File dump=new File(output,WSUtilities.extractResourceID(epr)+".workassignment-address");
			FileWriter fw=new FileWriter(dump);
			fw.write(epr.toString()+"\n");
			fw.close();
			verbose("Wrote work assignment address to "+dump.getAbsolutePath());
		}
		catch(Exception e){
			error("Could not write workflow ID file.",e);
		}
	}


	@Override
	public String getName() {
		return "broker-run";
	}

	@Override
	public String getArgumentList(){
		return "[<job-file>]";
	}

	@Override
	public String getSynopsis(){
		return "Runs a job through the Chemomentum service orchestrator. " +
		"The job definition (in UCC's JSON format) is read from <job-file>." +
		"A descriptor file will be written that can be used " +
		"later with other ucc commands.";
	}

	@Override
	public String getDescription(){
		return "submit a work assignment to a service orchestrator";
	}
	@Override
	public String getCommandGroup(){
		return "Workflow";
	}


	public QName getQName(){
		return IServiceOrchestrator.PORT;
	}

	public String getServiceName(){
		return "Service orchestrator";
	}

	public String getServiceDetails(EndpointReferenceType epr, IClientProperties security){
		return null;
	}

}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-workflow/src/main/java/de/fzj/unicore/ucc/workflow/SubmitWorkAssignment.java <==
package de.fzj.unicore.ucc.workflow;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Random;

import javax.security.auth.x500.X500Principal;
import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.chemomentum.common.ws.IServiceOrchestrator;
import org.chemomentum.workassignment.xmlbeans.GetWorkAssignmentStatusRequestDocument;
import org.chemomentum.workassignment.xmlbeans.GetWorkAssignmentStatusResponseDocument;
import org.chemomentum.workassignment.xmlbeans.StatusType;
import org.chemomentum.workassignment.xmlbeans.SubmitWorkAssignmentRequestDocument;
import org.chemomentum.workassignment.xmlbeans.WorkAssignmentType;
import org.chemomentum.workassignment.xmlbeans.WorkDocument.Work;
import org.chemomentum.workflow.callback.xmlbeans.ExecutionResourcesDocument.ExecutionResources;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.actions.Resolve;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.ucc.util.Builder;
import de.fzj.unicore.ucc.util.FileDownloader;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import eu.unicore.security.xfireutil.client.UnicoreXFireClientFactory;
import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * submit a work assignment to the UNICORE service orchestrator (broker)
 * 
 * @author schuller
 */
public class SubmitWorkAssignment extends WorkflowSystemSubmission implements IServiceInfoProvider{

	protected String workflowName;

	protected IServiceOrchestrator wsc;

	protected EndpointReferenceType servorchEPR;

	protected EndpointReferenceType jobEPR;

	protected String waID;

	protected boolean async=false;

	@Override
	public void process(){
		super.process();

		workflowName=getCommandLine().getOptionValue(OPT_NAME);
		if(getCommandLine().hasOption(OPT_MODE)){
			async=true;
		}
		waID=WSUtilities.newUniqueID();
		try{
			findSite();
			initBuilder(waID);
			run();
		}catch(Exception e){
			error("",e);
			endProcessing(1);
		}
	}

	protected void findSite(){
		String addr=null;
		try{
			List<EndpointReferenceType>available=registry.listAccessibleServices(IServiceOrchestrator.PORT);
			for(EndpointReferenceType epr: available){
				if (siteName!=null){
					addr=epr.getAddress().getStringValue();
					if(addr.contains(siteName)){
						servorchEPR=epr;
						setReceiver();
						wsc=new UnicoreXFireClientFactory(securityProperties).createPlainWSProxy(IServiceOrchestrator.class, addr);
						verbose("Using service <"+addr+"> at requested site <"+siteName+">");
						return;
					}
				}
			}
			int i=0;
			if(available.size()==0){
				message("No service orchestrator service available!");
				endProcessing(1);
			}
			else{
				//select one
				i=new Random().nextInt(available.size());
				servorchEPR=available.get(i);
				setReceiver();
				addr=servorchEPR.getAddress().getStringValue();
				wsc=new UnicoreXFireClientFactory(securityProperties).createPlainWSProxy(IServiceOrchestrator.class, addr);
			}
			verbose("Selected workflow service at "+addr);
		}
		catch(Exception e){
			error("Can't find a workflow service.",e);
			endProcessing();
		}
	}

	private void setReceiver(){
		String dn=null;
		try{
			dn=WSUtilities.extractServerIDFromEPR(servorchEPR);
			if(dn!=null){
				securityProperties.getETDSettings().setReceiver(new X500Principal(dn));
			}
		}catch(IllegalArgumentException e){
			//dn is invalid
			try{
				logger.warn("Invalid server name <"+dn+"> found in EPR "+servorchEPR.getAddress().getStringValue()+". Not an X500 name.");
			}catch(NullPointerException e1){}
		}

	}

	protected void run() throws Exception{
		//Submit work assignment
		SubmitWorkAssignmentRequestDocument in=SubmitWorkAssignmentRequestDocument.Factory.newInstance();
		WorkAssignmentType wa=in.addNewSubmitWorkAssignmentRequest().addNewWorkAssignment();
		Work work=wa.addNewWork();
		work.setJobDefinition(builder.getJob().getJobDefinition());
		wa.setParent("Submitted_by_UCC");
		wa.setId(waID);
		if(builder.getImports().size()>0){
			createWorkflowDataStorage();
			wa.setStorageEPR(storageEPR);
			try{
				uploadLocalData(builder, waID);
			}catch(IOException ex){
				error("Can't upload local files.",ex);
				endProcessing(1);
			}
		}
		
		logger.debug("Submitting "+String.valueOf(in));
		
		wsc.submitWorkAssignment(in);

		if(!async){
			
			try{
				waitForFinish();
			}catch(Exception ex){
				error("Work assignment did not finish successfully",ex);
				endProcessing(1);
			}
			//get output files
			if(builder.getExports().size()>0){
				for(FileDownloader exp: builder.getExports()){
					performExportToLocal(exp);
				}
			}
		}
	}

	protected void performExportToLocal(FileDownloader e){
		String name=waID+"_"+e.getTo();
		File out=new File(name);
		if(!out.isAbsolute()){
			out=new File(output,name);
		}
		try{
			Location l=Resolve.resolve(e.getFrom(), registry, securityProperties, this);
			EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
			epr.addNewAddress().setStringValue(l.getSmsEpr());
			StorageClient sms=new StorageClient(l.getSmsEpr(),epr,securityProperties);
			FileDownloader e1=new FileDownloader(l.getName(),out.getAbsolutePath(),e.getMode());
			e1.perform(sms, this);
		}
		catch(Exception ex){
			error("Could not perform export "+e.getFrom()+"->"+e.getTo(),ex);
		}
	}

	protected void waitForFinish()throws Exception{
		verbose("Waiting for work assignment to finish...");
		StatusType.Enum s=null;
		boolean havePrintedJobEPR=false;
		GetWorkAssignmentStatusResponseDocument res=null;
		do{
			GetWorkAssignmentStatusRequestDocument gs=GetWorkAssignmentStatusRequestDocument.Factory.newInstance();
			gs.addNewGetWorkAssignmentStatusRequest().setId(waID);
			res=wsc.getWorkAssignmentStatus(gs);
			StatusType.Enum newStatus=res.getGetWorkAssignmentStatusResponse().getStatusResult();
			if(!newStatus.equals(s)){
				s=newStatus;
				message(String.valueOf(s));
			}
			ExecutionResources er=res.getGetWorkAssignmentStatusResponse().getExecutionResources();
			if(er!=null){
				jobEPR=er.getJob();
				if(jobEPR!=null && !havePrintedJobEPR){
					message("Job submitted as: "+er.getJob().getAddress().getStringValue());
					havePrintedJobEPR=true;
				}
			}
			if(StatusType.FAILURE.equals(s) || StatusType.SUCCESS.equals(s))break;
			Thread.sleep(2000);
		}while(true);
		message("Work assignment finished: "+s);
	}


	private void initBuilder(String uniqueID) throws Exception {
		if(getCommandLine().getArgs().length<1){
			error("Please supply the name of the job file,", new IllegalArgumentException());
			endProcessing(1);
		}
		String filename=getCommandLine().getArgs()[1];
		verbose("Loading job from <"+filename+">");
		FileInputStream fis=new FileInputStream(new File(filename).getAbsolutePath());
		ByteArrayOutputStream bos=new ByteArrayOutputStream();
		int b=0;
		while( (b=fis.read())!=-1){
			bos.write(b);
		}
		String job=bos.toString();
		if(fixFilenames){
			job=writeNewIDs(job, uniqueID);
		}
		builder=new Builder(job);
	}


	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_MODE_LONG)
				.withDescription("Run asynchronously")
				.isRequired(false)
				.create(OPT_MODE)
		);


	}

	protected void writeIDFile(EndpointReferenceType epr){
		try{
			File dump=new File(output,WSUtilities.extractResourceID(epr)+".workassignment-address");
			FileWriter fw=new FileWriter(dump);
			fw.write(epr.toString()+"\n");
			fw.close();
			verbose("Wrote work assignment address to "+dump.getAbsolutePath());
		}
		catch(Exception e){
			error("Could not write workflow ID file.",e);
		}
	}

	@Override
	public String getName() {
		return "broker-run";
	}

	@Override
	public String getArgumentList(){
		return "[<job-file>]";
	}

	@Override
	public String getSynopsis(){
		return "Runs a job through the Chemomentum service orchestrator. " +
		"The job definition (in UCC's JSON format) is read from <job-file>." +
		"A descriptor file will be written that can be used " +
		"later with other ucc commands.";
	}

	@Override
	public String getDescription(){
		return "submit a work assignment to a service orchestrator";
	}
	@Override
	public String getCommandGroup(){
		return "Workflow";
	}

	@Override
	public QName getQName(){
		return IServiceOrchestrator.PORT;
	}

	@Override
	public String getServiceName(){
		return "Service orchestrator";
	}

	@Override
	public String getServiceDetails(EndpointReferenceType epr, IClientConfiguration security){
		return null;
	}

}
